@@ -1,72 +1,16 @@
 #!/usr/bin/env python
-from __future__ import print_function
-from __future__ import division
-
+import sys, json, yaml, os, re, importlib
 from collections import defaultdict , deque
-import sys, json, yaml, os, random, re ,copy, importlib, mmap
-from bson.objectid import ObjectId
 from time import time
 from datetime import timedelta as td, datetime as dt
-
 from pymongo.collection import Collection as mgcoll
-nolibs = []
-
-try:
-    import mysql.connector
-except:
-    nolibs.append( 'mysql-connector' )
-
-class JSONEncoder(json.JSONEncoder):
-    def default(self, o):
-        if isinstance(o, ObjectId):
-            return str(o)
-        if isinstance(o, deque):
-            return list(o)
-
-        try:
-            return json.JSONEncoder.default(self, o)
-        except:
-            return str(o)
-
-def loader(nd):
-    nam, ext = os.path.splitext( nd )
-
-    if not os.path.exists( nd ):
-        file = open( nd, "w+" )
-        file.write('{}')
-        file.close()
-
-    file = open( nd, "r+" )
-    text = file.read()
-    file.close()
-
-    data = {}
-    if text.strip() == '':
-        return data
-    elif ext == '.yaml':
-        data = yaml.safe_load( text )
-    elif ext == '.json':
-        data = json.loads(text )
-    elif text.strip() == '':
-        data = {}
-
-    return data
+from .helpers import loader, jsonencoder
 
 def to_jict(prev):
     nd = jict()
     for k,i in prev.items():
         if isinstance(i,dict):
             nd[k] = to_jict(i)
-
-        # elif isinstance(i,list):
-        #     nl = []
-        #     for l in i:
-        #         if isinstance(l,dict):
-        #             nl.append(to_jict(l))
-        #         else:
-        #             nl.append(l)
-        #     nd[k] = nl
-
         else:
             nd[k] = i
     return nd
@@ -80,6 +24,16 @@ class jict( defaultdict ):
             dt = to_jict(nd)
             return dt
 
+        # elif isinstance( nd, list ):
+        #     jct = jict()
+        #     for i,e in enumerate( nd ):
+        #         if isinstance( e, dict ):
+        #             e = to_jict(e)
+        #         jct[i] = e
+        #
+        #     jct.generator = list
+        #     return jct
+
         elif isinstance( nd , mgcoll ):
             jct = jict()
             k = nd.find({'key':extra})
@@ -90,51 +44,55 @@ class jict( defaultdict ):
             jct.storepath = extra
             return jct
         elif isinstance( nd, str ):
-            try:
-                if len(nd) >= 5 and nd[-5:] in [ '.yaml' , '.json' ]:
-                    if ( nd[:6] == 'shm//:' or nd[:6] == 'set://' ) and nd[-5:] == '.json':
-                        nd = nd[6:]
-                        dt = to_jict( loader(nd) )
-                        dt.storepath = nd
+            append = [ '.yaml' , '.json', '.list', '.env', '.env.example']
+            # prepend= [ 'shm//:', 'set://' ]
+            dt,transf,prepsf,file,size = None,'','',False,len(nd)
 
-                        f = open( nd, "r+b" )
-                        dt.file = mmap.mmap( f.fileno() , 0 )
-                        return dt
+            for x in append:
+                xs = len(x)
+                if xs >= size: continue
+                if nd[-xs:]==x:
+                    file,transf=True,x
+                    break
 
-                    if not os.path.exists( nd ):
-                        open( nd , 'w+' ).close()
+            # for x in prepend:
+            #     xs = len(x)
+            #     if xs => size: continue
+            #     if nd[:xs]==x:
+            #         prepsf,nd=x,nd[xs:]
+            #         break
 
-                    dt = to_jict( loader(nd) )
-                    dt.storepath = nd
+            if file:
+                if not os.path.isfile( nd ):
+                    if not os.path.exists((dr:=os.path.dirname(nd))):
+                        os.makedirs(dr)
+                    f = open( nd , 'w+' ); f.write("{}"); f.close()
+                    transf = ''
 
-                elif nd[-5:] == '.list':
-                    file = open( nd, "r+" )
-                    text = file.read()
-                    file.close()
-                    ls = []
-                    for x in text.split('\n'):
-                        if x == '': continue
-                        ls.append(x)
-
-                    return ls
-                elif nd[-4:] == '.env' or '.env.example' in nd:
-                    file = open( nd, "r+" )
-                    text = file.read()
-                    file.close()
-                    jct = jict()
-                    for x in text.split('\n'):
-                        if x == '': continue
-                        if '=' in x:
-                            fields = x.split('=')
-                            if len(fields) == 2:
-                                k,v = fields
-                                jct[k] = v
-
-                    jct.storepath = nd
-                    return jct
+            if transf in [ '.yaml' , '.json' ]:
+                da = loader( nd , transf )
+                dt = None
+                if isinstance(da,dict):
+                    dt = to_jict(da)
+                elif isinstance( da, list ):
+                    dt = jict()
+                    for i,e in enumerate( da ):
+                        if isinstance( e, dict ):
+                            e = to_jict(e)
+                        dt[i] = e
+                    dt.generator = list
                 else:
-                    dt = to_jict( json.loads( nd ) )
-            except Exception as e:
+                    dt = jict()
+                dt.storepath = nd
+            elif transf in ['.env','.env.example','.list']:
+                lst = [x for x in open(nd,"r+").read().split('\n') if x !='']
+                if transf == '.list':
+                    dt = lst
+                else:
+                    jct = jict({(x:=y.split('='))[0]:x[1]for y in lst if'='in y})
+                    jct.storepath = nd
+                    dt = jct
+            else:
                 dt = jict()
 
             return dt
@@ -145,28 +103,37 @@ class jict( defaultdict ):
         self.factory = jict
         defaultdict.__init__( self, self.factory )
 
-    # def __iter__(self):
-    #     started = False
-    #
-    #     if self.generator != None:
-    #         if not started:
-    #             started = True
-    #             yield jict(self.dict())
-    #
-    #         for x in self.generator:
-    #             yield to_jict(x)
-
-    def has(self, args ):
-        keys= self.keys()
-        for x in args:
-            if x not in keys: return False
-        return True
+        # if self.generator == list:
+        #     @property
+        #     def avg(self):
+        #         return self.sum / self.len
+        #     @property
+        #     def max(self):
+        #         suma = 0
+        #         for x in self: suma += self[x]
+        #         return max(self)
+        #     @property
+        #     def min(self):
+        #         return min(self)
+        #     @property
+        #     def sum(self):
+        #         suma = 0
+        #         for x in self: suma += self[x]
+        #         return suma
+        #     @property
+        #     def len(self):
+        #         return len(self.keys())
+        #
+        #     self.avg = avg
+        #     self.max = max
+        #     self.min = min
+        #     self.sum = sum
+        #     self.len = len
 
     # creates a default valuef for the key if doesn't has one
     def init(self,key, deft ):
         if key in self.keys():
             return self[key]
-
         self[key] = deft
         return self[key]
 
@@ -188,11 +155,12 @@ class jict( defaultdict ):
             except:
                 pass
 
-
         self[key] = deft
         return self[key]
 
     def strptime(self,key,pattern):
+        if key not in self:
+            return None
         return dt.strptime( self[key] ,pattern )
 
     def strptimestamp(self,key,pattern):
@@ -245,6 +213,16 @@ class jict( defaultdict ):
         self[key] = val if val < self[key] else self[key]
         return self[key]
 
+    def dropk(self,*args,as_type='',deep=False):
+        for x in args:
+            if x in self:
+                del self[x]
+            elif isinstance(self[x],jict) and deep:
+                self[x].dropk(*args,as_type='',deep=deep)
+
+        if as_type == 'dict':
+            return self.dict()
+
     def drop(self ,target = None, as_type='', deep = False ):
 
         def ittrlis(val,tar):
@@ -309,7 +287,6 @@ class jict( defaultdict ):
                 self[replace] = self.pop(target)
 
     def replace(self, target , replacef = None ):
-
         if replacef == None and isinstance(target,dict):
             for k in target.keys():
                 self.replace(k,target[k])
@@ -346,17 +323,17 @@ class jict( defaultdict ):
                 else:
                     self[k] = replacef
 
-    def _ittrlist(self,lst,k,luky=True ):
+    def _ittrlist(self,lst,k,v,luky=True ):
         found = []
         for x in lst:
             if isinstance(x,list):
-                found += self._ittrlist(x,k,luky)
+                found += self._ittrlist(x,k,v,luky)
                 continue
             elif isinstance(x,dict) or isinstance(x,jict):
-                found += self._ittrdict(x,k,luky)
+                found += self._ittrdict(x,k,v,luky)
         return found
 
-    def _ittrdict(self,dic,k ,luky = True ):
+    def _ittrdict(self,dic,k,v,luky = True ):
         found = []
         for x in dic.keys():
             val = dic[x]
@@ -367,16 +344,16 @@ class jict( defaultdict ):
                 continue
 
             if isinstance(val, list):
-                found += self._ittrlist(val,k,luky)
+                found += self._ittrlist(val,k,v,luky)
                 continue
 
             elif isinstance(val,dict) or isinstance(val,jict):
-                found += self._ittrdict(val,k,luky)
+                found += self._ittrdict(val,k,v,luky)
 
         return found
 
-    def get(self,key,luky = True ):
-        ret = self._ittrdict(self,key, luky )
+    def get(self,key,val=None,luky = True ):
+        ret = self._ittrdict(self, key, val , luky )
 
         if len(ret) == 1:
             return ret[-1]
@@ -397,63 +374,16 @@ class jict( defaultdict ):
                 plain_dict[ key ] = value
         return plain_dict
 
-    def random(self):
-        for x in range( random.randint(0, 9) ):
-            self[x] = x
-            if random.randint(0, 1) == 0:
-                self[x] = [ f for f in range(random.randint(0, 10)) ]
-
     def __str__(self):
         return self.json()
 
     def json(self,indent=2):
-        if indent == 0: return json.dumps( self.dict() , cls= JSONEncoder )
-        return json.dumps( self.dict() , indent = indent , cls= JSONEncoder )
+        if indent == 0: return json.dumps( self.dict() , cls= jsonencoder )
+        return json.dumps( self.dict() , indent = indent , cls= jsonencoder )
 
     def yaml(self):
         return yaml.dump(yaml.full_load( self.json(0) ), default_flow_style=False)
 
-    def __getitem__(self,key):
-        jct = super( defaultdict, self ).__getitem__(key)
-
-        if hasattr( self , 'skig' ):
-            return jct
-
-        if hasattr( self , 'file' ):
-            mm = self.file
-            mm.seek( 0 )
-            text = mm.read(mm.size())
-            data = json.loads( text )
-            dt = jict( data )
-            val = dt[key]
-
-            setattr(self,'skip',True)
-            self[key] = val
-            del self.skip
-            return val
-
-        return jct
-
-    def __setitem__( self , key , data ):
-        super(defaultdict, self).__setitem__( key , data )
-
-        # stops the infinite set
-        if hasattr( self , 'skip' ):
-            return
-
-        if hasattr( self , 'file' ):
-            mm = self.file
-
-            setattr(self,'skig',True)
-            ns = bytes( self.json( 0 ) , 'utf-8')
-            del self.skig
-            mm.seek( 0 )
-            if mm.size() != len( ns ):
-                mm.resize( len( ns ) )
-
-            mm.seek( 0 )
-            mm.write(ns)
-
     def deque(self,key,data,maxlen = 5 ):
         if not isinstance(self[key], deque ):
             if isinstance(self[key], list ):
@@ -462,29 +392,9 @@ class jict( defaultdict ):
                 self[key] = deque(maxlen = maxlen)
 
         self[key].append( data )
-        if hasattr( self , 'file' ):
-            mm = self.file
-
-            setattr(self,'skig',True)
-            ns = bytes( self.json( 0 ) , 'utf-8')
-            del self.skig
-
-            mm.seek( 0 )
-            if mm.size() != len( ns ):
-                mm.resize( len( ns ) )
-
-            mm.seek( 0 )
-            mm.write(ns)
-
-    def close(self):
-        if hasattr( self , 'file' ):
-            self.file.flush()
-            self.file.close()
-
-    def save(self, name = None, tp = None , shm = '' ):
-        if hasattr( self , 'file' ):
-            return
 
+    def save(self, name = None ):
+        # generators
         if isinstance(self.generator, mgcoll ):
             if '_id' not in self.keys():
                 k = self.generator.find({'key':self.storepath})
@@ -497,54 +407,36 @@ class jict( defaultdict ):
                 self.generator.update_one(
                     { '_id':self['_id'] },{ '$set':{'data':self.json()} }
                 )
+            return self
 
-            return
-        # if name != None:
-        #     valid = ['sql://']
-        #
-        #     for x in valid:
-        #         if len(name) >= len(x) and name[:len(x)] == x:
-        #             if name[:len(x)] == 'sql://':
-        #                 self.sql_store(name[len(x):])
-        #                 return
+        if name != None:
+            self.storepath = name
 
-        self.storepath = name if name != None else self.storepath \
-                    if self.storepath != None else 'jict.json'
+        if self.storepath == None:
+            return self
 
-        nam, ext = os.path.splitext( self.storepath )
-        tp = ext if tp == None else '.' + tp if not tp[0] == '.' else tp
-        self.storepath = nam + tp
+        if not os.path.isfile( self.storepath ):
+            dr = os.path.dirname(self.storepath)
+            if not os.path.exists(dr):
+                os.makedirs(dr)
+
+        transf = ''
+        size = len(self.storepath)
+        for x in ['.list','.env','.env.example','.yaml','.json']:
+            xs = len(x)
+            if xs >= size: continue
+            if self.storepath[-xs:]==x:
+                transf=x
+                break
 
         f = open(self.storepath, "w+")
-        if tp == '.example' or self.storepath[-4:] == '.env':
-            txt = ''
-            for x in self.keys():
-                if isinstance( self[x] ,str ):
-                    txt += f'{x}={self[x]}\n'
-            f.write( txt )
-        elif tp == '.yaml':
+        if transf in ['.example','.env']:
+            f.write( '\n'.join([f'{x}='+str(self[x]) for x in self.keys()]) )
+        elif transf == '.list':
+            f.write( '\n'.join([str(self[x]) for x in self.keys()]) )
+        elif transf == '.yaml':
             f.write( self.yaml() )
         else:
             f.write( self.json() )
 
         f.close()
-
-    # def sql_store(self,db):
-    #     connection = sqlconnect()
-    #
-    #     skip = ['columns']
-    #
-    #     for table in self.keys():
-    #         lines = self[table]
-    #
-    #         if table in skip:
-    #             continue
-    #
-    #         for line in lines:
-    #             print( line )
-    #
-    #     cursor.close()
-    #     connection.close()
-    #
-    #     del cursor
-    #     del connection
